
#include "aux0/cayley-dickson-mult.h"

#include "bits/graycode.h"
#include "bits/parity.h"

#include "fxtio.h"
#include "demo/nextarg.h"
#include "fxttypes.h"

#include "jjassert.h"

//% Multiplication table with hypercomplex numbers (Cayley-Dickson construction)


//#define TIMING // uncomment to disable printing

//#define USE_REC_OPT // define to use optimization with recursive version
// note: makes routine slower with large n

int
main(int argc, char **argv)
{
//    gen_table();

    ulong ldn = 5;
    NXARG(ldn, "Multiplication table for (2**ldn)-ions");
    const ulong n = 1UL << ldn;
    bool q = 0;
    NXARG(q, "Whether to include indices of components");
    // Note: with TIMING  q==1 ==> iterative routine, else use recursion

#ifdef TIMING

    int x = 0;
    if ( q )
    {
        cout << "iterative routine:" << endl;
        for (ulong r=0; r<n; ++r)
            for (ulong c=0; c<n; ++c) { x ^= CD_sign_it(r, c, n); }
    }
    else
    {
        cout << "recursive routine:" << endl;
        for (ulong r=0; r<n; ++r)
            for (ulong c=0; c<n; ++c) { x ^= CD_sign_rec(r, c, n); }
    }
    return x;

#else

    for (ulong r=0; r<n; ++r)
    {
        cout << setw(3) << r << ":  ";
        for (ulong c=0; c<n; ++c)
        {
            int s = CD_sign_it(r, c, n);
            if ( 0==q )  cout << " " << (s<0 ? '-' : '+');
            else
            {
                cout << setw(3) << (r^c) << (s<0 ? '-' : '+');
            }

            jjassert( s == CD_sign_rec(r,c,n) );
        }
        cout << endl;
    }

    return 0;
#endif
}
// -------------------------

/*

Timing:

 time ./bin 12 1
iterative routine:
./bin 12 1  1.46s user 0.00s system 99% cpu 1.464 total
 ==> 2^24/1.46  == 11,491,243 per second

 time ./bin 12 0
recursive routine:
./bin 12 0  1.88s user 0.01s system 99% cpu 1.884 total
 ==> 2^24/1.88  == 8,924,051 per second


time ./bin 13 1
iterative routine:
./bin 13 1  6.45s user 0.01s system 99% cpu 6.462 total
 ==> 2^26/6.45  == 10,404,475 per second

 time ./bin 13 0
./bin 13 0  8.30s user 0.02s system 100% cpu 8.316 total
 ==> 2^26/8.30  == 8,085,405 per second


 time ./bin 14 1
iterative routine:
./bin 14 1  28.26s user 0.03s system 100% cpu 28.288 total
 ==> 2^28/28.26  == 9,498,777 per second

 time ./bin 14 0
recursive routine:
./bin 14 0  36.38s user 0.06s system 99% cpu 36.439 total
 ==> 2^28/36.38  == 7,378,654 per second


BENCHARGS=13 1
BENCHARGS=13 0

*/


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/arith"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/arith/cayley-dickson-demo.cc"
/// make-target2: "1demo DSRC=demo/arith/cayley-dickson-demo.cc DEMOFLAGS=-DTIMING"
/// End:

